package de.ullisroboterseite.ursanalogclock;

import com.google.appinventor.components.annotations.*;
import com.google.appinventor.components.common.*;
import com.google.appinventor.components.runtime.*;
import com.google.appinventor.components.runtime.util.*;

import java.lang.reflect.*;

import android.util.Log;

import android.view.View;
import android.view.ViewGroup;

public class Helper {
  static final String LOG_TAG = UrsViewBase.LOG_TAG;

  /**
   * Ermittelt per Reflection die übergeordnete Komponente eines Steuerelements
   * @param viewComponent Komponente, deren Container-Komponente ermittelt werden soll.
   * @return Die übergeordnete Komponente. Ist entweder vom Typ Screen, ReplForm, ...Arrangement oder TableArrangement
   * @throws Exception Der Zugriff auf das Feld 'container' war nicht möglich.
   */
  static public ComponentContainer getControlsParent(AndroidViewComponent viewComponent) throws Exception {
    // Die übergebene Komponente ist ggf. indirekt von AndroidViewComponent abgeleitet.
    // Deshalb solange in der Klassenhierarchie nach oben steigen, bis man bei AndroidViewComponent angekommen ist.
    Class placeholderClass = viewComponent.getClass();

    while (placeholderClass != AndroidViewComponent.class)
      placeholderClass = placeholderClass.getSuperclass();

    Field f = placeholderClass.getDeclaredField("container");
    f.setAccessible(true); // ermöglicht den Zugriff auf private Variablen
    return (ComponentContainer) f.get(viewComponent); // ist entweder Screen, ReplForm, ...Arrangement oder TableArrangement
  }

  /**
   * Ermittelt die integrierte ViewGroup des dem Platzhalter übergeordneten Containers
   * @param placeholder Der zu Grunde liegende Platzhalter
   * @return True, wenn es sich bei dem Container um ein TableArrangement handelt.
   * @throws Exception Die internen Felder sind nich zugreifbar
   */
  static public android.view.ViewGroup getContainersViewGroup(AndroidViewComponent placeholder) throws Exception {
    ComponentContainer placeholderParent = Helper.getControlsParent(placeholder);
    Class placeholderParentClass = placeholderParent.getClass();

    // bei ...Arrangement ist eine Zwischenableitung HVArrangement vorhanden
    // bei der Form liegt die abgeleitete Klasse ScreenX vor
    if (placeholderParent instanceof HVArrangement || placeholderParent instanceof Form) {
      placeholderParentClass = placeholderParentClass.getSuperclass();
    }

    // Im Companion ist der Typ ScreenX vom Typ ReplForm abgeleitet, der nochmals von Form abgeleitet ist
    if (placeholderParent instanceof ReplForm) {
      placeholderParentClass = placeholderParentClass.getSuperclass();
    }

    Field f = placeholderParentClass.getDeclaredField("viewLayout");
    f.setAccessible(true);
    Object viewLayout = f.get(placeholderParent);

    android.view.ViewGroup layoutManager;
    // viewLayout kann vom Typ LinearLayout oder TableLayout sein
    // beide besitzen die Methode "getLayoutManager"
    try {
      layoutManager = ((LinearLayout) viewLayout).getLayoutManager();
      // ist vom Typ "LinearLayout$1", eine Ableitung von android.view.ViewGroup
    } catch (Exception e) {
      try {
        layoutManager = ((TableLayout) viewLayout).getLayoutManager();
      } catch (Exception ex) {
        throw ex;
      }
      // nichts zu tun
    }
    return layoutManager;
  }
}